searchData={"items":[{"type":"module","title":"AhoCorasearch","doc":"Documentation for `AhoCorasearch`.","ref":"AhoCorasearch.html"},{"type":"function","title":"AhoCorasearch.build_tree/2","doc":"Builds the tree needed for efficient string searching","ref":"AhoCorasearch.html#build_tree/2"},{"type":"function","title":"Examples - AhoCorasearch.build_tree/2","doc":"```\niex> AhoCorasearch.build_tree([{\"abc\", 1}, {\"abc\", 2}, {\"def\", 36}, {\"xyz\", 72}])\n{:ok, \"#AhoCorasearch.Tree \"}\n\niex> AhoCorasearch.build_tree([{\"abc\", 1}, {\"def\", 2}], unique: true, match_kind: :standard, insensitive: false)\n{:ok, \"#AhoCorasearch.UniqueTree \"}\n\niex> AhoCorasearch.build_tree([{\"abc\", 1}, {\"abc\", 2}, {\"def\", 36}, {\"xyz\", 72}], unique: true)\n{:error, :not_unique}\n```\n\nThe options it accepts are:\n`unique` - Whether the keys in the pattern list are unique or not. This determines what the result tuple will look like. `{integer, integer, integer}` for unique trees, and `{integer, integer, list(integer)}` for non-unique trees. Defaults to `false`.\n`insensitive` - If the matching should be case insensitive. Insensitivity is acheived by downcasing the patterns, and also downcasing the search text. This has a minor performance impact, but in most cases a negligible one. Defaults to `true`.\n`match_kind` - There are 3 different matching options, but they must be chosen a tree building time, not at search time. The options are `standard`, `leftmost_longest`, or `leftmost_first`. Defaults to `leftmost_longest`.\n\nGiven the following patterns:\n```\n[\n  {\"headphone\", 0},\n  {\"head\", 1},\n  {\"phone\", 2},\n  {\"pine\", 3},\n  {\"cone\", 4},\n  {\"pinecone\", 5}\n]\n```\nSearching with `AhoCorasearch.search(tree, \"headphones and pinecones\")`\n  standard:\n  will return `[{0, 4, [1]}, {4, 9, [2]}, {15, 19, [3]}, {19, 23, [4]}]`\n  which translates to `[\"head\", \"phone\", \"pine\", \"cone\"]`\n\n  leftmost_longest:\n  will return `[{0, 9, [0]}, {15, 23, [5]}]`\n  which translates to `[\"headphone\", \"pinecone\"]`\n\n  leftmost_first:\n  will return `[{0, 9, [0]}, {15, 19, [3]}, {19, 23, [4]}]`\n  which translates to `[\"headphone\", \"pine\", \"cone\"]`\n  (notice that `first` here is first in the pattern list, not first in the search string).\n\nThe examples above demonstrate the return value for a non-unique tree. The results is `{start_index, end_index, ids}`. Not that the indexes are in bytes, so you'll want to use `:binary.part/3` to extract substrings, not `String.slice/3`. The ID part must be an integer, but it does not need to be unique. Multiple keys can use the same ID. If multiple patterns are given with the same key and ID, the result will be repeated, for example: `[{34, 35, [1, 1]}]`. IDs are returned in the same order they are given, so for patterns: `[{\"a\", 3}, {\"a\", 1}, {\"a\", 2}, {\"a\", 1}]`, an \"a\" in the search text would give the result: `{X, Y, [3, 1, 2, 1]}`","ref":"AhoCorasearch.html#build_tree/2-examples"},{"type":"function","title":"AhoCorasearch.heap_bytes/1","doc":"Returns the number of bytes that the tree is using on the native/Rust side.\nThis does not account for any memory being stored on the Beam side, which hold the duplicate map\nTotal size(beam + native) can be approximated with:\n`:erts_debug.flat_size(tree) + AhoCorasearch.heap_bytes(tree)`\n\nThis function is only intended for use in development/debugging. I don't know the implications(performance or safety) of using it in production.","ref":"AhoCorasearch.html#heap_bytes/1"},{"type":"function","title":"AhoCorasearch.search/3","doc":"Searches the given string, against the given tree. This is one of two available ways to search. This is the faster of the two methods, but also might give more unexpected data. It will return matches anywhere they appear in the string(assuming the rules still abide by the tree's match configuration), but it has no notion of words. So partial and subwords will match. If you're looking for only full word matches, look at the `AhoCorasearch.word_search/3`, or use the option `word_search: true` which will filter out any matches that are not complete words(starting and ending at a word boundary).\n\nThe inputs are:\n`tree`: `AhoCorasearch.Tree.t()` - The tree that was build with `build_tree/2-3`\n`string`: `String.t()` - The string that will be searched against the tree\n`opts`:\n  `overlap: boolean`(default: false) - This argument is ONLY used if the tree type is `standard`. For trees with another `match_kind` it is ignored.\n  `word_search: boolean`(default: false) - Applies an additional filtering step to remove any matches that aren't complete words\n\nThe result is a list of matches, either in the form `{integer, integer, integer}` for unique tree, or `{integer, integer, list(integer)}` for non-unique trees. The values are: `{start_index, end_index, ID}`. ID is whatever you passed in as the pattern(`[{\"string\", ID}, ...]`).\n\nThe `start_index` and `end_index` are in BYTE values, not character or codepoints, so be sure you operate on the string with them by using `:binary.part/3` rather than `String.slice/3`.\n\nreturns a list of all the matches in the input string.\nleftmost, longest matches\nWhile this handles unicode optimally, the return start/stop values are\nin bytes. i.e. use `:binary.part/3` rather than `String.slice/3`","ref":"AhoCorasearch.html#search/3"},{"type":"function","title":"AhoCorasearch.word_search/3","doc":"This is a convenience function for doing a word based search. Internally it just sets the `word_search` option to true for `search/3`, so either can be used as they're the same.","ref":"AhoCorasearch.html#word_search/3"},{"type":"type","title":"AhoCorasearch.id/0","doc":"","ref":"AhoCorasearch.html#t:id/0"},{"type":"type","title":"AhoCorasearch.match/0","doc":"","ref":"AhoCorasearch.html#t:match/0"},{"type":"type","title":"AhoCorasearch.match_kind/0","doc":"","ref":"AhoCorasearch.html#t:match_kind/0"},{"type":"type","title":"AhoCorasearch.matches/0","doc":"","ref":"AhoCorasearch.html#t:matches/0"},{"type":"type","title":"AhoCorasearch.options/0","doc":"","ref":"AhoCorasearch.html#t:options/0"},{"type":"type","title":"AhoCorasearch.pattern/0","doc":"","ref":"AhoCorasearch.html#t:pattern/0"},{"type":"type","title":"AhoCorasearch.patterns/0","doc":"","ref":"AhoCorasearch.html#t:patterns/0"},{"type":"type","title":"AhoCorasearch.search_opts/0","doc":"","ref":"AhoCorasearch.html#t:search_opts/0"},{"type":"type","title":"AhoCorasearch.start/0","doc":"","ref":"AhoCorasearch.html#t:start/0"},{"type":"type","title":"AhoCorasearch.stop/0","doc":"","ref":"AhoCorasearch.html#t:stop/0"},{"type":"type","title":"AhoCorasearch.tree/0","doc":"","ref":"AhoCorasearch.html#t:tree/0"},{"type":"module","title":"AhoCorasearch.Native","doc":"","ref":"AhoCorasearch.Native.html"},{"type":"function","title":"AhoCorasearch.Native.build_tree/2","doc":"","ref":"AhoCorasearch.Native.html#build_tree/2"},{"type":"function","title":"AhoCorasearch.Native.downcase/1","doc":"","ref":"AhoCorasearch.Native.html#downcase/1"},{"type":"function","title":"AhoCorasearch.Native.find_iter/2","doc":"","ref":"AhoCorasearch.Native.html#find_iter/2"},{"type":"function","title":"AhoCorasearch.Native.find_overlapping_iter/2","doc":"","ref":"AhoCorasearch.Native.html#find_overlapping_iter/2"},{"type":"function","title":"AhoCorasearch.Native.get_match_kind/1","doc":"","ref":"AhoCorasearch.Native.html#get_match_kind/1"},{"type":"function","title":"AhoCorasearch.Native.leftmost_find_iter/2","doc":"","ref":"AhoCorasearch.Native.html#leftmost_find_iter/2"},{"type":"function","title":"AhoCorasearch.Native.tree_heap_bytes/1","doc":"","ref":"AhoCorasearch.Native.html#tree_heap_bytes/1"},{"type":"module","title":"AhoCorasearch.Tree","doc":"","ref":"AhoCorasearch.Tree.html"},{"type":"function","title":"AhoCorasearch.Tree.heap_bytes/1","doc":"","ref":"AhoCorasearch.Tree.html#heap_bytes/1"},{"type":"function","title":"AhoCorasearch.Tree.new/2","doc":"","ref":"AhoCorasearch.Tree.html#new/2"},{"type":"function","title":"AhoCorasearch.Tree.search/3","doc":"","ref":"AhoCorasearch.Tree.html#search/3"},{"type":"type","title":"AhoCorasearch.Tree.match_kind/0","doc":"","ref":"AhoCorasearch.Tree.html#t:match_kind/0"},{"type":"type","title":"AhoCorasearch.Tree.t/0","doc":"","ref":"AhoCorasearch.Tree.html#t:t/0"},{"type":"module","title":"AhoCorasearch.TreeBuilder","doc":"","ref":"AhoCorasearch.TreeBuilder.html"},{"type":"function","title":"AhoCorasearch.TreeBuilder.build_tree/2","doc":"","ref":"AhoCorasearch.TreeBuilder.html#build_tree/2"},{"type":"function","title":"AhoCorasearch.TreeBuilder.check_unique/2","doc":"","ref":"AhoCorasearch.TreeBuilder.html#check_unique/2"},{"type":"function","title":"AhoCorasearch.TreeBuilder.new/2","doc":"","ref":"AhoCorasearch.TreeBuilder.html#new/2"},{"type":"function","title":"AhoCorasearch.TreeBuilder.split_patterns/1","doc":"","ref":"AhoCorasearch.TreeBuilder.html#split_patterns/1"},{"type":"module","title":"AhoCorasearch.WordSearch","doc":"WordSearch enables a more refined search mechanism that takes into account full words.\n\nFor example \"Earphones and pinecones\" using the regular(faster) `AhoCorasearch.search/2` function will match(depending on settings and input): `[\"ear\", \"phone\", \"earphone\", \"and\", \"pine\", \"cone\", \"cones\", \"pinecone\", \"pinecones\"]`. Which might be what you want. This module will help if what you'd expect to match is `[\"earphones\", \"and\", \"pinecones\"]`(assuming all those words are in your tree).\n\nIt's a slower search method than the regular search, by a reasonable factor.","ref":"AhoCorasearch.WordSearch.html"},{"type":"function","title":"AhoCorasearch.WordSearch.filter/2","doc":"","ref":"AhoCorasearch.WordSearch.html#filter/2"},{"type":"module","title":"Typespecs","doc":"","ref":"Typespecs.html"},{"type":"extras","title":"Usage","doc":"# Usage","ref":"usage.html"},{"type":"extras","title":"What is it? - Usage","doc":"AhoCorasearch is an Elixir wrapping of the Rust library, [Daachorse](https://github.com/daac-tools/daachorse), which is a fast implementation of the Aho-Corasick string searching algorithm, specifically the double-array version of Aho-Corasick.","ref":"usage.html#what-is-it"},{"type":"extras","title":"Why would I want to use it? - Usage","doc":"Aho-Corasick is a very fast string searching algorithm provided that the keywords you want to search for in the novel text, are known ahead of time(so that the intial setup can be done at startup, or amortized over multiple searches). For example if you're searching for a fixed list of keywords in multiple documents, then this will be well suited for your needs. If on the other hand you're looking to search a dozen documents, but with a different set of keywords for each document, then Aho-Corasick might not be the best algorithm for what you're looking for.","ref":"usage.html#why-would-i-want-to-use-it"},{"type":"extras","title":"Basic Usage - Usage","doc":"First you must create the tree that will be used for the optimized string searching.\n\nThe input is a list of tuples, the first element a string, the second is an integer ID that you can later use to corelate the matches.\n```\npatterns = [{\"is\", 0}, {\"was\", 1}, {\"an\", 2}, {\"am\", 3}]\n\n{:ok, tree} = AhoCorasearch.build_tree(patterns)\n```\n\nThis function returns an opaque handle to a Rust resource. You can store it just like you would with a Ref or PID. It can be stored in ETS, a GenServer, an Agent, etc. But similar to a PID, it can't be stored in module attributes or anything compile time as it's  a reference to a runtime object in memory. To use it, you pass it to the `search/2-3` function.\n\n```\nAhoCorasearch.search(tree, \"what is this example you're showing me?\")\n\n# => [{5, 7, [0]}, {10, 12, [0]}, {15, 17, [3]}]\n```\n\nThe format of the results is the following: \n```\n{\n  integer: start_index, \n  integer: end_index, \n  integer | list(integer): ID | IDs\n}\n```\n\n> ##","ref":"usage.html#basic-usage"},{"type":"extras","title":"Unique vs non-unique(default) {: .info} - Usage","doc":">\n> Passing `unique: true` to `build_tree/3` will give you single integer IDs in the results\n> \n> [{5, 7, 0}, ...]\n> \n> By default(without passing `unique`, or by passing `unique: false`) you can have a pattern like this `[{\"a\", 1}, {\"a\", 2}]`\n> \n> The result for \"a\" would be `[{X, Y, [1, 2]}, ...]`\n\nSo we can use it as follows:\n\n```\nstring = \"what is this example you're showing me?\"\nresults = AhoCorasearch.search(tree, \"what is this example you're showing me?\")\n# => [{5, 7, [0]}, {10, 12, [0]}, {15, 17, [3]}]\nEnum.map(results, fn {start, stop, _id} ->\n  :binary.part(string, start, stop - start)\nend)\n# => [\"is\", \"is\", \"am\"]\n```\n\n`\"what [is] th[is] ex[am]ple you're showing me?\"`\n\nThere are different matching types you can use to vary the results. These are all required to be set during the tree building phase, not the searching phase.\n\n### `standard`\n\n`standard` will return the first match it comes across(not overlapping by default, but can be set to overlapping with a option on the `search/3` function `[overlap: true]`). I know it's confusing, but `standard` is not the default match_kind, `leftmost_longest` is, an explanation for why is further below.\n\nSo, `[\"pine\", \"cone\", \"econ\"]` against `\"pinecone\"` would match `\"pine\"` and `\"cone\"`, or all 3 when using `overlap: true`.\n\n```\niex(127)> {:ok, tree} = AhoCorasearch.build_tree(Enum.with_index([\"pine\", \"cones\", \"pinecones\"]), match_kind: :standard)\n{:ok, \"#AhoCorasearch.Tree \"}\niex(128)> AhoCorasearch.search(tree, \"pinecones\")\n[{0, 4, [0]}, {4, 9, [1]}]\niex(129)> AhoCorasearch.search(tree, \"pinecones\", overlap: true)\n[{0, 4, [0]}, {0, 9, [2]}, {4, 9, [1]}]\n```\n\n### `leftmost_longest`(default match_kind)\n\n`leftmost_longest` will match, you guessed it, the rightmost, shortest. No, it will match whatever it finds first in the string(leftmost) that is the longest(of all matches starting at that location). To demonstrate we'll use the patterns `[\"pin\", \"pine\", \"inecones\", \"con\", \"cone\", \"cones\"]`\n\n\n```\niex(137)> {:ok, tree} = AhoCorasearch.build_tree(Enum.with_index([\"pin\", \"pine\", \"inecones\", \"con\", \"cone\", \"cones\"]))\n{:ok, \"#AhoCorasearch.Tree \"}\niex(128)> AhoCorasearch.search(tree, \"pinecones\")\n[{0, 4, [1]}, {4, 9, [4]}]\niex(140)> |> Enum.map(fn {start, stop, _id} -> :binary.part(\"pinecones\", start, stop - start) end)\n[\"pine\", \"cones\"]\n```\n\nSo `pine` matched because it was leftmost. `inecones` didn't because although it was longer than `pine`, it was futher right, and overlapping. `con`, `cone`, `cones` started at the same position, but since `cones` is longer, it matched. _If_ the search string _didn't_ have the last `s` in it, then `cone` would have matched.\n\n`leftmost_longest` does not accept an `overlap` option, because an overlapping `leftmost_longest` is really just `stardard` with overlapping turned on.\n\n\n### `leftmost_first`\n\n`leftmost_first` will match similar to `leftmost_longest`, except instead of length, it prioritizes the order which the pattern appeared in the input patterns when building the tree. So if you wanted to search for `cone`, `con`, and `cones`.\n\n```\niex(144)> {:ok, tree} = AhoCorasearch.build_tree(Enum.with_index([\"cone\", \"con\", \"cones\"]), match_kind: :leftmost_first)\n{:ok, \"#AhoCorasearch.Tree \"}\niex(145)> AhoCorasearch.search(tree, \"pinecones\")\n[{4, 8, [0]}]\n```\n\nHere we see `\"cone\"` as our match, because it was the the \"first registered\" of all the matches starting at `\"c\"`.","ref":"usage.html#unique-vs-non-unique-default-info"},{"type":"extras","title":"Whole word matching - Usage","doc":"You may have noticed all the matches so far will match any subsection of the string, regardless of word boundary. This is the default setting and the way that Aho-Corasick works. If what you want is word boundary matching words, so you want to match \"pine\" if it appears as \"pine\", or \"pine.\" or \"pine,\", but not \"pinecone\", then there is another search function you can use. It works exactly the same as the one above(except it's a bit slower), but it will filter out matches that aren't at word boundaries.\n\n```\niex(146)> {:ok, tree} = AhoCorasearch.build_tree(Enum.with_index([\"pine\"]), match_kind: :standard)\n{:ok, \"#AhoCorasearch.Tree \"}\niex(147)> AhoCorasearch.word_search(tree, \"pine, pine pine. pinecone conepine pine.\", overlap: true)\n[{0, 4, [0]}, {6, 10, [0]}, {11, 15, [0]}, {35, 39, [0]}]\n```\nNow we just see matches that happen at word boundaries. \n\n`\"[pine], [pine] [pine]. pinecone conepine [pine].\"`\n\n`AhoCorasearch.word_search/3` is just a convienience function for `AhoCorasearch.search(tree, string, word_search: true)`\n\nFor `:standard` match_kind, `overlap: true` is still a valid option here, because patterns can be multiple words. Patterns: `car`, `red car`, `red` would return all three when searched against `red car`, with `overlap: true`, because they all start/end at word boundaries.\n\nThe `AhoCorasearch.word_search/3`(or using `word_search: true`) function is a bit slower because it does a regex on the search string to find word boundaries, but it's still quite fast, and it should work for any unicode word boundaries, not just space, comma, period, etc.","ref":"usage.html#whole-word-matching"},{"type":"extras","title":"Unicode/chars - Usage","doc":"The searching is optimized to work on unicode characters. Even though the return results are in byte offsets, the following will work totally fine. \n\n```\npatterns = String.split(\"👨‍👩‍👧‍👦🏯👩🏻‍🎤🇨🇴☮👨🏼‍💻\", \"\", trim: true) |> Enum.with_index()\niex(156)> {:ok, tree} = AhoCorasearch.build_tree(patterns, match_kind: :standard)\n{:ok, \"#AhoCorasearch.Tree \"}\niex(157)> AhoCorasearch.search(tree, \"some 👨‍👩‍👧‍👦 more 🏯 text 👩🏻‍🎤 in 🇨🇴 between ☮ here 👨🏼‍💻.\")\n[\n  {5, 30, [0]},\n  {36, 40, [1]},\n  {46, 61, [2]},\n  {65, 73, [3]},\n  {82, 85, [4]},\n  {91, 106, [5]}\n]\n```\n\nYou'll notice the offsets are much larger now, 25 bytes for the first emoji. So make sure you're always extracting from the string with `:binary.part/3`\n\n> ##","ref":"usage.html#unicode-chars"},{"type":"extras","title":"Do this {: .tip} - Usage","doc":">\n> ```\n> string = \"some 👨‍👩‍👧‍👦 more 🏯 text 👩🏻‍🎤 in 🇨🇴 between ☮ here 👨🏼‍💻.\"\n> results = [\n>   {5, 30, [0]},\n>   {36, 40, [1]},\n>   {46, 61, [2]},\n>   {65, 73, [3]},\n>   {82, 85, [4]},\n>   {91, 106, [5]}\n> ]\n> iex(161)> Enum.map(results, fn {start, stop, _id} ->\n> ...(161)>   :binary.part(string, start, stop - start)\n> ...(161)> end)\n> [\"👨‍👩‍👧‍👦\", \"🏯\", \"👩🏻‍🎤\", \"🇨🇴\", \"☮\",\n>  \"👨🏼‍💻\"]\n> ```\n\nElixir's `String.slice/3` is character based, and will give you totally fine results in ascii text where byte/codepoint/char 1 == 1 == 1, but you'll quickly know something went wrong if you try it for anything where that's not true. For example with the first emoji (byte/codepoint/char) 25 == 7 == 1. The 25 is the number the response results are using.\n\n\n> ##","ref":"usage.html#do-this-tip"},{"type":"extras","title":"Don't do this {: .warning} - Usage","doc":">\n> ```\n> iex(164)> Enum.map(results, fn {start, stop, _id} ->\n> ...(164)>   String.slice(string, start, stop - start)\n> ...(164)> end)\n> [\"👨‍👩‍👧‍👦 more 🏯 text 👩🏻‍🎤 in 🇨🇴 betw\",\n>  \"here\", \"\", \"\", \"\", \"\"]\n> ```\n\nOne other note, by default the searching is case insensitive, but this can be changed with the `insensitive` option on `build_tree/2`. This is different from how Aho-Corasick usually works, and is acheived by merely downcasing all input patterns, and downcasing the search text before searching. The downcasing is also done by a Rust NIF for performance reasons. I'm not 100% sure if there are any differences between Rust's `to_lowercase()` and Elixir's `String.downcase/1`, but I did run a test against Unicode's `UnicodeData.txt`, and found no differences(aside from Rust being faster).\n\nThe defaults in general are a bit opinionated, mostly because this code was part of a codebase that needed that specific functionality, and when I decided to extract this code into a separate library for releasing as OSS, I didn't want to potentially introduce bugs into the original codebase by changing the default behavior of this code. If you want something that most closely resembles a textbook version of Aho-Corasick, use the following options: `AhoCorasearch.build_tree(patterns, match_kind: :standard, unique: true, insensitive: false)`","ref":"usage.html#don-t-do-this-warning"},{"type":"extras","title":"Performance - Usage","doc":"All benchmarks below can be run with `mix test --include benchmark`.\n\nBecause ultimately this code is wrapping a library written in Rust, it's very performant. On short strings(~100 characters), i've seen it hit 1.2M searches per second. The longest text benchmark I did was with a tree of ~150k English words, against the full bible, and Benchee showed IPS of ~27/s. In general, Aho-Corasick scales with the length of the search string, and the number of matches(longer string, and/or more matches in a string == slower execution), not the number of keywords in the tree.\n\nThe memory performance is also quite good, because the algorithm uses a dense tree structure, new entries can share nodes with previous entries(if there's character overlap), so the cost of each new entry trends downward as the tree grows. Also repeated keys are nearly free(except the small amount to hold the integer ID), because only one instance of that string gets sent to the Rust code. Below you'll find an assortment of benchmarks. As always with benchmarks, they only test for the exact settings/circumstances they test for, on the exact day/machine they were tested on, which may have nothing in common with your system/requirements/use case, so if performance is a major concern, do some benchmarking that more closesly matches your use case.\n\nMetadata for all performance results:\n\n```\nOperating System: macOS\nCPU Information: Apple M1 Pro\nNumber of Available Cores: 10\nAvailable memory: 16 GB\nElixir 1.15.3\nErlang 26.0\n\nBenchmark suite executing with the following configuration:\nwarmup: 0 ns\ntime: 5 s\nmemory time: 0 ns\nreduction time: 0 ns\nparallel: 1\ninputs: none specified\n```\n\n##","ref":"usage.html#performance"},{"type":"extras","title":"Testing build_tree with 1000 patterns: - Usage","doc":"```\nName                                     ips        average  deviation         median         99th %\naho_corasearch_build_tree             204.30        4.89 ms     ±3.06%        4.86 ms        5.43 ms\naho_corasick_erlang_build_tree         10.56       94.70 ms    ±16.19%       88.98 ms      178.38 ms\naho_corasick_elixir_build_tree          2.14      467.53 ms    ±12.60%      445.57 ms      547.47 ms\n\nComparison:\naho_corasearch_build_tree             204.30\naho_corasick_erlang_build_tree         10.56 - 19.35x slower +89.80 ms\naho_corasick_elixir_build_tree          2.14 - 95.52x slower +462.63 ms\n```\n\n##","ref":"usage.html#testing-build_tree-with-1000-patterns"},{"type":"extras","title":"Testing search this lib vs other erlang/elixir libs, long text. byte_size: 642,000 - Usage","doc":"```\nName                                 ips        average  deviation         median         99th %\naho_corasearch_search             201.03        4.97 ms     ±3.21%        4.93 ms        5.59 ms\naho_corasick_erlang_search          7.29      137.10 ms     ±2.37%      135.72 ms      148.90 ms\naho_corasick_elixir_search          0.70     1433.25 ms     ±1.31%     1435.43 ms     1453.33 ms\n\nComparison:\naho_corasearch_search             201.03\naho_corasick_erlang_search          7.29 - 27.56x slower +132.13 ms\naho_corasick_elixir_search          0.70 - 288.13x slower +1428.28 ms\n```\n\n##","ref":"usage.html#testing-search-this-lib-vs-other-erlang-elixir-libs-long-text-byte_size-642-000"},{"type":"extras","title":"Testing search(this lib vs others), short text. byte_size: 510 - Usage","doc":"```\nName                                 ips        average  deviation         median         99th %\naho_corasearch_search           380.23 K        2.63 μs   ±488.58%        2.54 μs        2.83 μs\naho_corasick_erlang_search       10.32 K       96.89 μs    ±40.05%          95 μs      118.79 μs\naho_corasick_elixir_search        1.27 K      786.02 μs     ±5.04%      776.88 μs      927.89 μs\n\nComparison:\naho_corasearch_search           380.23 K\naho_corasick_erlang_search       10.32 K - 36.84x slower +94.26 μs\naho_corasick_elixir_search        1.27 K - 298.87x slower +783.39 μs\n```\n\n##","ref":"usage.html#testing-search-this-lib-vs-others-short-text-byte_size-510"},{"type":"extras","title":"Testing search (this lib only) different match kinds + unique/non-unique - Usage","doc":"```\nName                                           ips        average  deviation         median         99th %\naho_corasearch_standard_uniq              376.73 K        2.65 μs   ±319.76%        2.54 μs        2.88 μs\naho_corasearch_standard_uniq_overlap      370.57 K        2.70 μs   ±322.54%        2.58 μs        2.92 μs\naho_corasearch_left_first_uniq            362.39 K        2.76 μs   ±307.24%        2.67 μs           3 μs\naho_corasearch_standard                   362.29 K        2.76 μs   ±310.07%        2.63 μs        3.08 μs\naho_corasearch_left_long_uniq             361.91 K        2.76 μs   ±312.74%        2.67 μs        2.96 μs\naho_corasearch_standard_overlap           355.19 K        2.82 μs   ±325.55%        2.71 μs        3.13 μs\naho_corasearch_left_first                 345.11 K        2.90 μs   ±302.28%        2.79 μs        3.21 μs\naho_corasearch_left_long                  343.39 K        2.91 μs   ±325.44%        2.79 μs        3.21 μs\n\nComparison:\naho_corasearch_standard_uniq              376.73 K\naho_corasearch_standard_uniq_overlap      370.57 K - 1.02x slower +0.0441 μs\naho_corasearch_left_first_uniq            362.39 K - 1.04x slower +0.105 μs\naho_corasearch_standard                   362.29 K - 1.04x slower +0.106 μs\naho_corasearch_left_long_uniq             361.91 K - 1.04x slower +0.109 μs\naho_corasearch_standard_overlap           355.19 K - 1.06x slower +0.161 μs\naho_corasearch_left_first                 345.11 K - 1.09x slower +0.24 μs\naho_corasearch_left_long                  343.39 K - 1.10x slower +0.26 μs\n```\n\n##","ref":"usage.html#testing-search-this-lib-only-different-match-kinds-unique-non-unique"},{"type":"extras","title":"Testing AhoCorasearch.search/3 vs AhoCorasearch.word_search/3 short text byte_size: 510 - Usage","doc":"```\nName                                           ips        average  deviation         median         99th %\naho_corasearch_standard                   366.25 K        2.73 μs   ±351.96%        2.63 μs           3 μs\naho_corasearch_left_long                  345.57 K        2.89 μs   ±374.89%        2.79 μs        3.13 μs\naho_corasearch_standard_word_search        69.25 K       14.44 μs    ±14.76%       14.17 μs       18.54 μs\naho_corasearch_left_long_word_search       68.68 K       14.56 μs    ±18.71%       14.33 μs       19.38 μs\n\nComparison:\naho_corasearch_standard                   366.25 K\naho_corasearch_left_long                  345.57 K - 1.06x slower +0.163 μs\naho_corasearch_standard_word_search        69.25 K - 5.29x slower +11.71 μs\naho_corasearch_left_long_word_search       68.68 K - 5.33x slower +11.83 μs\n```\n\n##","ref":"usage.html#testing-ahocorasearch-search-3-vs-ahocorasearch-word_search-3-short-text-byte_size-510"},{"type":"extras","title":"Testing AhoCorasearch.search/3 vs AhoCorasearch.word_search/3 long text byte_size: 642,000 - Usage","doc":"```\nName                                                     ips        average  deviation         median         99th %\naho_corasearch_left_long_long_text                    201.33        4.97 ms     ±2.33%        4.93 ms        5.36 ms\naho_corasearch_standard_long_text                     199.55        5.01 ms     ±1.75%        4.98 ms        5.41 ms\naho_corasearch_word_search_left_long_long_text         63.94       15.64 ms     ±1.29%       15.59 ms       16.55 ms\naho_corasearch_standard_word_search_long_text          63.61       15.72 ms     ±0.72%       15.72 ms       16.13 ms\n\nComparison:\naho_corasearch_left_long_long_text                    201.33\naho_corasearch_standard_long_text                     199.55 - 1.01x slower +0.0444 ms\naho_corasearch_word_search_left_long_long_text         63.94 - 3.15x slower +10.67 ms\naho_corasearch_standard_word_search_long_text          63.61 - 3.17x slower +10.75 ms\n```\n\n#","ref":"usage.html#testing-ahocorasearch-search-3-vs-ahocorasearch-word_search-3-long-text-byte_size-642-000"},{"type":"extras","title":"What are the most performant settings? - Usage","doc":"If you really want to eek out the most performance this library can give, the following settings: `[unique: true, insensitive: false]` will bypass a lot of the additional work that gets done in elixir. These settings will generally only be different by a meaningful amount if you are expecting a lot of matches(because `unique: false` does a map lookup for each match), and don't care about the searching being case insensitive(this allows the lib to skip doing a downcase pass on the search string, which saves time for long search strings).\n\n\"Best\" case settings:\n```\nAhoCorasearch.build_tree(patterns, unique: true, insensitive: false, match_kind: :standard)\n```\n\n\"Worst\" case settings:\n```\nAhoCorasearch.build_tree(patterns, match_kind: :standard)\n```\n\n(the `match_kind` doesn't actually matter much for performance generally. I'm using `standard` because along with `overlap: true` it returns the most matches, which does change performance a bit)\n\n##","ref":"usage.html#what-are-the-most-performant-settings"},{"type":"extras","title":"Testing AhoCorasearch best case vs worst case: long text(byte_size: ~640k, num_matches: ~1M): - Usage","doc":"```\nName                                          ips        average  deviation         median         99th %\naho_corasearch_best_case_long_text          38.56       25.93 ms    ±24.64%       23.31 ms       35.94 ms\naho_corasearch_worst_case_long_text         20.77       48.15 ms     ±8.50%       47.58 ms       82.42 ms\n\nComparison:\naho_corasearch_best_case_long_text          38.56\naho_corasearch_worst_case_long_text         20.77 - 1.86x slower +22.21 ms\n```\n\n##","ref":"usage.html#testing-ahocorasearch-best-case-vs-worst-case-long-text-byte_size-640k-num_matches-1m"},{"type":"extras","title":"Testing AhoCorasearch best case vs worst case: short text(byte_size: 100, num_matches: 166): - Usage","doc":"```\nName                                           ips        average  deviation         median         99th %\naho_corasearch_best_case_short_text       384.43 K        2.60 μs   ±299.87%        2.50 μs        3.25 μs\naho_corasearch_worst_case_short_text      156.65 K        6.38 μs   ±129.82%        5.13 μs       18.79 μs\n\nComparison:\naho_corasearch_best_case_short_text       384.43 K\naho_corasearch_worst_case_short_text      156.65 K - 2.45x slower +3.78 μs\n```\n\n##","ref":"usage.html#testing-ahocorasearch-best-case-vs-worst-case-short-text-byte_size-100-num_matches-166"},{"type":"extras","title":"General performance usage - Usage","doc":"Overall the tree generated by `build_tree/2` is safe and fast to pass between processes, as well as safe to use from multiple procesess concurrently. Because the tree itself is stored in Rust, passing the tree from one process to another does not require copying the underlying Rust memory, only the reference handle which is tiny. The only edge case would be if you had huge numbers of duplicated strings in the tree, then there is some memory stored in the Beam side to hold the mapping of IDs, but this is a `%{integer => list(integer)}` map, so is really not a concern unless the numbers get massive. As always benchmarking your particular use case is the best way to know.\n\n#","ref":"usage.html#general-performance-usage"},{"type":"extras","title":"Oddities / Known bugs - Usage","doc":"Currently the ability to pass in non-unique patterns is handled purely in Elixir as the underlying Rust lib requires unique patterns, so the Elixir code rolls up any duplicates and passes a single value to Rust in order to build the tree, and then resolves these back into multiple IDs after a search is done. Generally this causes no problems, but be aware that the logic merely aggregates the duplicates under the first key/ID pair, which can cause some unexpected behaviour in the following edge case(note the different return values for the \"b\" match based on which ID it overlaps with):\n\n```\niex(53)> {:ok, tree} = AhoCorasearch.build_tree([{\"a\", 0}, {\"a\", 1}, {\"b\", 1}])\niex(54)> AhoCorasearch.search(tree, \"ab\")\n[{0, 1, [0, 1]}, {1, 2, [1]}]\n```\n```\niex(56)> {:ok, tree} = AhoCorasearch.build_tree([{\"a\", 0}, {\"a\", 1}, {\"b\", 0}])\niex(57)> AhoCorasearch.search(tree, \"ab\")\n[{0, 1, [0, 1]}, {1, 2, [0, 1]}]\n```\n\nYou'll see that in the first example, even though a/1 and b/1 overlapped, because \"b\" was the first key to have ID: 1(a/1 was rolled into a/0), it returns only [1] in the results. When \"b\" is changed to 0, it's ID is now the same as the ID being used for \"a\", so \"a\" and \"b\" will have the same IDs([0, 1]). If you want some specific functionality that this conflicts with, you're best bet is passing `unique: true` to `AhoCorasearch.build_tree/2` and managing the ID resolution yourself.\n\n\nAlso, this code was my first foray in the Rust language, and my first time using Rustler to create Elixir NIFs, so there's likely a few things that could be done differently/better with that code. That being said it's fairly short code, with really no complex logic, and has been very stable in the time i've used it. However, if you have any suggestions for improvements, please let me know and I'll be happy to take a look into it.","ref":"usage.html#oddities-known-bugs"}],"content_type":"text/markdown"}